#!/usr/bin/env python3

# External command, intended to be called with meson.add_dist_script() in meson.build

#                        argv[1]         argv[2]        argv[3:]
# extra-dist-cmd.py <root_source_dir> <root_build_dir> <no_dist>...

# <no_dist> Zero or more names (relative to MESON_PROJECT_DIST_ROOT)
#           of files and directories that shall not be distributed.

# Meson does not preserve timestamps on distributed files.
# But this script preserves the timestamps on libstdc++.tag.

import os
import sys
import subprocess
import shutil

root_source_dir = sys.argv[1]
root_build_dir = sys.argv[2]

# Make a ChangeLog file for distribution.
cmd = [
  'git',
  '--git-dir=' + os.path.join(root_source_dir, '.git'),
  '--work-tree=' + root_source_dir,
  'log',
  '--no-merges',
  '--date=short',
  '--max-count=200',
  '--pretty=tformat:%cd  %an  <%ae>%n%n  %s%n%w(0,0,2)%+b',
]
# MESON_PROJECT_DIST_ROOT is set only if meson.version() >= 0.58.0.
project_dist_root = os.getenv('MESON_PROJECT_DIST_ROOT', os.getenv('MESON_DIST_ROOT'))
logfilename = os.path.join(project_dist_root, 'ChangeLog')
with open(logfilename, mode='w', encoding='utf-8') as logfile:
  result = subprocess.run(cmd, stdout=logfile)
  if result.returncode:
    sys.exit(result.returncode)

# Distribute the libstdc++.tag file in addition to the files in the local git clone.
# shutil.copy2() copies timestamps and some other file metadata.
shutil.copy2(os.path.join(root_build_dir, 'libstdc++.tag'),
             os.path.join(project_dist_root, 'doctags'))

# Remove specified files and directories from the MESON_PROJECT_DIST_ROOT directory.
for rel_path in sys.argv[3:]:
  abs_path = os.path.join(project_dist_root, rel_path)
  if os.path.isfile(abs_path):
    os.remove(abs_path)
  elif os.path.isdir(abs_path):
    shutil.rmtree(abs_path, ignore_errors=True)
  else:
    # Ignore non-existent files and directories.
    print('--- Info:', abs_path, 'not found, not removed.')



#!/usr/bin/env python3

# External command, intended to be called with meson.add_install_script() in meson.build

#                           argv[1]
# extra-install-cmd.py <aclocal_macrodir>

import os
import sys
import subprocess

if not (os.getenv('DESTDIR') or os.getenv('MESON_INSTALL_QUIET')):
  # Inform the installer that M4 macro files installed in a directory
  # not known to aclocal will not be picked up automatically.
  # (Starting with Python 3.7 text=True is a more understandable equivalent to
  # universal_newlines=True. Let's use only features in Python 3.5.)
  try:
    result = subprocess.run(['aclocal', '--print-ac-dir'],
                            stdout=subprocess.PIPE, stderr=subprocess.DEVNULL,
                            universal_newlines=True)
    acdir = result.stdout
    aclocal_path = os.getenv('ACLOCAL_PATH')
    # acdir and aclocal_path can be sequences of os.pathsep-separated paths.
    # Merge them to one sequence with leading and trailing os.pathsep.
    # os.pathsep is ':' for Linux, ';' for Windows.
    acdirs = os.pathsep
    if aclocal_path:
      acdirs += aclocal_path + os.pathsep
    if acdir:
      acdirs += acdir.rstrip('\n') + os.pathsep

    if (os.pathsep + sys.argv[1] + os.pathsep) not in acdirs:
      # f'''.....''' would require Python 3.6. Avoid it.
      print('''\
                NOTE
                ----
(If you don't intend to use Autotools, you can ignore this note.)
The mm-common Autoconf macro files have been installed in a different
directory than the system aclocal directory. In order for the installed
macros to be found, it may be necessary to add the mm-common include
path to the ACLOCAL_PATH environment variable:
  ACLOCAL_PATH="$ACLOCAL_PATH:{}"
  export ACLOCAL_PATH'''.format(sys.argv[1])
      )
  except FileNotFoundError:
    # https://mail.gnome.org/archives/gtkmm-list/2021-April/msg00005.html
    # On Windows, subprocess.run() may fail to find aclocal even if
    # the calling meson.build file has found it. Ignore this case.
    pass
sys.exit(0)



#!/usr/bin/env python3

# External command, intended to be called with run_command() in meson.build.

#                argv[1]    argv[2]    argv[3:]
# copy-files.py <from_dir> <to_dir> <file_names>...

import os
import sys
import shutil

# <from_dir> is an absolute or relative path of the directory to copy from.
# <to_dir> is an absolute or relative path of the directory to copy to.
from_dir_root = sys.argv[1]
to_dir_root = sys.argv[2]

# Copy some files if they exist in from_dir, but not in the destination
# directory, or if they are not up to date in the destination directory.
# (The term "source directory" is avoided here, because from_dir might not
# be what Meson calls a source directory as opposed to a build directory.)

for file in sys.argv[3:]:
  from_file = os.path.join(from_dir_root, file)
  to_file = os.path.join(to_dir_root, file)
  if os.path.isfile(from_file) and ((not os.path.isfile(to_file))
     or (os.stat(from_file).st_mtime > os.stat(to_file).st_mtime)):

    # Create the destination directory, if it does not exist.
    os.makedirs(os.path.dirname(to_file), exist_ok=True)

    # shutil.copy2() copies timestamps and some other file metadata.
    shutil.copy2(from_file, to_file)
sys.exit(0)
